/*
 * @(#)PCClock.java
 *
 * Summary: Platform dependent JNI glue code to set system clock for SetClock, version of Windows/NT.
 *
 * Copyright: (c) 1999-2009 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  7.5 2006-03-05 - reformat with IntelliJ and add Javadoc
 *  7.6 2007-06-14 - change return code type.
 *  7.7 2008-09-23 - fix problem with MS C++ run time library.
 */
package com.mindprod.pcclock;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.SimpleTimeZone;
import java.util.TimeZone;

/**
 * Platform dependent JNI glue code to set system clock for SetClock, version of Windows/NT.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 7.7 2008-09-23 - fix problem with MS C++ run time library.
 * @since 1999
 */
public final class PCClock
    {
    // ------------------------------ CONSTANTS ------------------------------

    /**
     * undisplayed copyright notice
     */
    public static final String EMBEDDED_COPYRIGHT =
            "copyright (c) 1999-2009 Roedy Green, Canadian Mind Products, http://mindprod.com";

    private static final String RELEASE_DATE = "2008-09-23";

    private static final String TITLE_STRING = "PCClock";

    private static final String VERSION_STRING = "7.7";

    // ------------------------------ FIELDS ------------------------------

    private static final TimeZone UTC = new SimpleTimeZone( 0, "UTC" );

    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * no arg constructor
     */
    public PCClock()
        {

        }

    /**
     * Set date and Time
     *
     * @param d contains UTC date/time
     *
     * @return true if all went ok
     */
    public boolean setClock( Date d )
        {
        // take Timestamp apart into fields
        Calendar c = new GregorianCalendar( UTC );
        c.setTime( d );
        int year = c.get( Calendar.YEAR );
        int month = c.get( Calendar.MONTH ) + 1;
        int day = c.get( Calendar.DAY_OF_MONTH );
        int hour = c.get( Calendar.HOUR_OF_DAY );
        int minute = c.get( Calendar.MINUTE );
        int second = c.get( Calendar.SECOND );
        int millis = c.get( Calendar.MILLISECOND );

        try
            {
            return nativeSetClock( year,
                    month,
                    day,
                    hour,
                    minute,
                    second,
                    millis );
            }
        catch ( Exception e )
            {
            System.out
                    .println(
                            "Unable to access PCClock.nativeSetClock method in nativepcclock.dll." );
            System.out.println( e.getMessage() );
            return false;
            }
        }

    // -------------------------- STATIC METHODS --------------------------

    static
        {
        try
            {
            // load nativepcclock.dll on path, set up by Java Web Start
            System.loadLibrary( "nativepcclock" );
            }
        catch ( Exception e )
            {
            System.out.println( "Unable to load nativepcclock.dll" );
            System.out.println( e.getMessage() );
            System.exit( 1 );
            }
        }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * nativeSetClock wants UTC time. natural year, month, day -- not zero based. return true if all went ok.
     */
    private native boolean nativeSetClock( int year,
                                           int month,
                                           int day,
                                           int hour,
                                           int minute,
                                           int second,
                                           int millis );
    }// end PCClock
